package ch.rakudave.jnetmap.controller;

import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuListener;

import ch.rakudave.jnetmap.util.Icons;
import ch.rakudave.jnetmap.util.Lang;
import ch.rakudave.jnetmap.util.Settings;

public class RecentlyOpened implements MenuListener {
	public static final int SIZE = 5;
	private static ArrayList<String> history = new ArrayList<String>(SIZE);
	private JMenu menu;
	
	public RecentlyOpened(JMenu parent) {
		menu = parent;
	}

	public static void load() {
		history.clear();
		for (int i = 0; i < SIZE; i++) {
			String path = Settings.get("recent."+i, "");
			if (!path.isEmpty()) history.add(path);
		}
	}
	
	public static void save() {
		for (int i = 0; i < SIZE; i++) {
			Settings.put("recent."+i, (i < history.size())?history.get(i):"");
		}
	}
	
	public static void put(String path) {
		history.remove(path);
		if (history.size() == SIZE) history.remove(0);
		history.add(path);
	}
	
	public static void clear() {
		history.clear();
	}

	@SuppressWarnings("serial")
	@Override
	public void menuSelected(MenuEvent e) {
		menu.removeAll();
		for (int i = history.size()-1; i >= 0; i--) {
			final File f = new File(history.get(i));
			menu.add(new JMenuItem(new AbstractAction(f.getName()) {
				@Override
				public void actionPerformed(ActionEvent e) {
					Actions.open(f, null);
				}
			}));
		}
		menu.add(new JSeparator());
		menu.add(new JMenuItem(new AbstractAction(Lang.get("menu.file.recent.clear"), Icons.get("remove")) {
				@Override
				public void actionPerformed(ActionEvent e) {
					history.clear();
				}
			}));
	}
	
	@Override
	public void menuDeselected(MenuEvent e) {}
	
	@Override
	public void menuCanceled(MenuEvent e) {}
}
